@@ -64,7 +64,7 @@ class OMI_NO2_KNMI_HDF_v2_0_preFeb2006_filetype():
                  'MeasurementQualityFlags' : 'Unitless',
                  'SlantColumnAmountNO2' : 'Molecules cm^-2 x 10^-15',
                  'SlantColumnAmountNO2Std' : 'Molecules cm^-2 x 10^-15',
-                 'Surface Albedo' : 'Unitless x 10,000',
+                 'SurfaceAlbedo' : 'Unitless x 10,000',
                  'TM4PressurelevelA' : 'Pa',
                  'TM4PressurelevelB' : 'Unitless', 
                  'TM4SurfacePressure' : 'hPa',
@@ -200,7 +200,7 @@ class OMI_NO2_KNMI_HDF_v2_0_postFeb2006_filetype():
                  'MeasurementQualityFlags' : 'Unitless',
                  'SlantColumnAmountNO2' : 'Molecules cm^-2 x 10^-15',
                  'SlantColumnAmountNO2Std' : 'Molecules cm^-2 x 10^-15',
-                 'Surface Albedo' : 'Unitless x 10,000',
+                 'SurfaceAlbedo' : 'Unitless x 10,000',
                  'TM4PressurelevelA' : 'Pa',
                  'TM4PressurelevelB' : 'Unitless', 
                  'TM4SurfacePressure' : 'hPa',
@@ -346,7 +346,7 @@ class OMI_NO2_NASA_HDF_v1_2_filetype():
                                     "not want to use corners, set to 'none'," \
                                     "and use nearest neighbor mapping.", 
                                     "dirPath")}
-    overallQualFlag = 'vcdQualityFlags'
+    overallQualFlag = 'VcdQualityFlags'
     cloudFrac = 'CloudFraction'
     solarZenithAngle = 'SolarZenithAngle'
     time = 'Time'
@@ -427,7 +427,7 @@ class OMI_NO2_NASA_HDF_v1_2_filetype():
                  'WavelengthRegistrationCheck' : 'nm',
                  'WavelengthRegistrationCheckStd' : 'nm',
                  'UnpolFldLatBandQualityFlags' : 'NoUnits',
-                 'vcdQualityFlags' : 'NoUnits' }
+                 'VcdQualityFlags' : 'NoUnits' }
 
     extraDimLabel = { 'Time' : 'none',
                  'Latitude' : 'none',
@@ -504,7 +504,7 @@ class OMI_NO2_NASA_HDF_v1_2_filetype():
                  'WavelengthRegistrationCheck' : 'WavelengthCheck',
                  'WavelengthRegistrationCheckStd' : 'WavelengthCheck',
                  'UnpolFldLatBandQualityFlags' : 'Latitude',
-                 'vcdQualityFlags' : 'none' }
+                 'VcdQualityFlags' : 'none' }
 
     extraDimSize = { 'Time' : 0,
                  'Latitude' : 0,
@@ -581,7 +581,7 @@ class OMI_NO2_NASA_HDF_v1_2_filetype():
                  'WavelengthRegistrationCheck' : 60,
                  'WavelengthRegistrationCheckStd' : 60,
                  'UnpolFldLatBandQualityFlags' : 180,
-                 'vcdQualityFlags' : 0 }
+                 'VcdQualityFlags' : 0 }
 
 class HDFmopittl2_generic_filetype():
     parser = "HDFmopittl2"
@@ -898,3 +898,352 @@ class MODIS_AOD_NASA_Collection_5_filetype():
                "Deep_Blue_Number_Pixels_Used_Land": ['3'],
                "Deep_Blue_Aerosol_Optical_Depth_550_Land_STD": [],
                "Deep_Blue_Aerosol_Optical_Depth_Land_STD": ['3']}
+class OMI_HCHO_NASA_HDF_v3_filetype():
+    parser = "HDFnasaomihchol2"
+    doutf = "OMIHCHO_netCDF_avg"
+    parserParms = {"cornerFileList":("To expedite file parsing time, the "   \
+                                     "list of files containing geolocation " \
+                                     "information for the four corners "     \
+                                     "(OMPIXCOR) may be passed in .  There " \
+                                     "must be one corner file for each of "  \
+                                     "the input files.", "list"),
+                   "cornerDir":("Path to the directory containing corner "\
+                                    "files with geolocation information for " \
+                                    "the four corners (OMPIXCOR). If you do " \
+                                    "not want to use corners, set to 'none'," \
+                                    "and use nearest neighbor mapping.", 
+                                    "dirPath")}
+    overallQualFlag = 'MainDataQualityFlag'
+    cloudFrac = 'AMFCloudFraction'
+    XtrackQualityFlags = 'XtrackQualityFlags'
+    solarZenithAngle = 'SolarZenithAngle'
+    time = 'Time'
+    longitude = 'Longitude'
+    pixIndXtrackAxis = 1
+    outUnits = { 'Time' : 'second',
+                 'Latitude' : 'degree',
+                 'Longitude' :'degree (-180 to 180)',
+                 'PixelCornerLongitudes' : 'degree (-180 to 180)',
+                 'PixelCornerLatitudes' : 'degree (-180 to 180)',                 	
+                 'SpacecraftAltitude' : 'meters',
+                 'SolarZenithAngle' : 'degree',
+                 'SolarAzimuthAngle' : 'degree',
+                 'ViewingZenithAngle' : 'degree',
+                 'ViewingAzimuthAngle' : 'degree',
+                 'GroundPixelQualityFlags' : 'NoUnits',
+                 'ColumnAmount' : 'molecules/cm^2',
+                 'ColumnAmountDestriped' : 'molecules/cm^2',
+                 'ColumnAmountUncertainty' : 'molecules/cm^2',
+                 'ClimatologyLevels' : 'hPa',
+                 'ColumnAmount' : 'molecules/cm^2',
+                 'ColumnAmountDestriped' : 'molecules/cm^2',
+                 'ColumnUncertainty' : 'molecules/cm^2',
+                 'GasProfile' : 'molecules/cm^2',
+                 'RadianceReferenceColumnAmount' : 'molecules/cm^2',
+                 'RadianceReferenceColumnUncertainty' : 'molecules/cm^2',
+                 'ReferenceSectorCorrectedVerticalColumn' : 'molecules/cm^2',
+                 'ScatteringWeights' : 'NoUnits',
+                 'SolarWavCalConvergenceFlag' : 'NoUnits',
+                 'AMFCloudFraction' : 'NoUnits',
+                 'AMFCloudPressure' : 'NoUnits',
+                 'AirMassFactor' : 'NoUnits',
+                 'AirMassFactorDiagnosticFlag' : 'NoUnits',
+                 'AirMassFactorGeometric' : 'NoUnits',
+                 'Albedo' : 'NoUnits',
+                 'SpacecraftAltitude' : 'meters',
+                 'TerrainHeight' : 'meters',
+                 'MainDataQualityFlag' : 'NoUnits',
+                 'FitConvergenceFlag' : 'NoUnits',
+                 'RadianceReferenceConvergenceFlag' : 'NoUnits',
+                 'RadianceWavCalConvergeFlag' : 'NoUnits'}
+
+    extraDimLabel = { 'Time' : 'none',
+                 'Latitude' : 'none',
+                 'Longitude' :'none',
+                 'PixelCornerLongitudes' : 'none',
+                 'PixelCornerLatitudes' : 'none',                 	
+                 'SpacecraftAltitude' : 'none',
+                 'SolarZenithAngle' : 'none',
+                 'SolarAzimuthAngle' : 'none',
+                 'ViewingZenithAngle' : 'none',
+                 'ViewingAzimuthAngle' : 'none',
+                 'GroundPixelQualityFlags' : 'none',
+                 'ColumnAmount' : 'none',
+                 'ColumnAmountDestriped' : 'none',
+                 'ColumnAmountUncertainty' : 'none',
+                 'ClimatologyLevels' : 'layer',
+                 'ColumnAmount' : 'none',
+                 'ColumnAmountDestriped' : 'none',
+                 'ColumnUncertainty' : 'none',
+                 'GasProfile' :'layer',
+                 'RadianceReferenceColumnAmount' : 'none',
+                 'RadianceReferenceColumnUncertainty' :'none',
+                 'ReferenceSectorCorrectedVerticalColumn' : 'none',
+                 'ScatteringWeights' : 'layer',
+                 'SolarWavCalConvergenceFlag' : 'none',
+                 'AMFCloudFraction' : 'none',
+                 'AMFCloudPressure' : 'none',
+                 'AirMassFactor' : 'none',
+                 'AirMassFactorDiagnosticFlag' : 'none',
+                 'AirMassFactorGeometric' : 'none',
+                 'Albedo' : 'none',
+                 'SpacecraftAltitude' : 'none',
+                 'TerrainHeight' : 'none',
+                 'MainDataQualityFlag' : 'none',
+                 'FitConvergenceFlag' : 'none',
+                 'RadianceReferenceConvergenceFlag' : 'none',
+                 'RadianceWavCalConvergeFlag' : 'none'}
+
+    extraDimSize = { 'Time' : 0,
+                 'Latitude' : 0,
+                 'Longitude' :0,
+                 'PixelCornerLongitudes' : 0,
+                 'PixelCornerLatitudes' : 0,                 	
+                 'SpacecraftAltitude' : 0,
+                 'SolarZenithAngle' : 0,
+                 'SolarAzimuthAngle' : 0,
+                 'ViewingZenithAngle' : 0,
+                 'ViewingAzimuthAngle' : 0,
+                 'GroundPixelQualityFlags' : 0,
+                 'ColumnAmount' : 0,
+                 'ColumnAmountDestriped' : 0,
+                 'ColumnAmountUncertainty' : 0,
+                 'ClimatologyLevels' : 47,
+                 'ColumnAmount' : 0,
+                 'ColumnAmountDestriped' : 0,
+                 'ColumnUncertainty' : 0,
+                 'GasProfile' :47,
+                 'RadianceReferenceColumnAmount' : 0,
+                 'RadianceReferenceColumnUncertainty' :0,
+                 'ReferenceSectorCorrectedVerticalColumn' : 0,
+                 'ScatteringWeights' : 47,
+                 'SolarWavCalConvergenceFlag' : 0,
+                 'AMFCloudFraction' : 0,
+                 'AMFCloudPressure' : 0,
+                 'AirMassFactor' : 0,
+                 'AirMassFactorDiagnosticFlag' : 0,
+                 'AirMassFactorGeometric' : 0,
+                 'Albedo' : 0,
+                 'SpacecraftAltitude' : 0,
+                 'TerrainHeight' : 0,
+                 'MainDataQualityFlag' : 0,
+                 'FitConvergenceFlag' : 0,
+                 'RadianceReferenceConvergenceFlag' : 0,
+                 'RadianceWavCalConvergeFlag' : 0}
+class OMI_NO2_NASA_HDF_v3_filetype():
+    parser = "HDFnasaomiv3l2"
+    doutf = "OMNO2e_netCDF_avg_v3"
+    parserParms = {"cornerFileList":("To expedite file parsing time, the "   \
+                                     "list of files containing geolocation " \
+                                     "information for the four corners "     \
+                                     "(OMPIXCOR) may be passed in .  There " \
+                                     "must be one corner file for each of "  \
+                                     "the input files.", "list"),
+                   "cornerDir":("Path to the directory containing corner "\
+                                    "files with geolocation information for " \
+                                    "the four corners (OMPIXCOR). If you do " \
+                                    "not want to use corners, set to 'none'," \
+                                    "and use nearest neighbor mapping.", 
+                                    "dirPath")}
+    overallQualFlag = 'VcdQualityFlags'
+    cloudFrac = 'CloudFraction'
+    XTrackQualityFlags = 'XTrackQualityFlags'
+    solarZenithAngle = 'SolarZenithAngle'
+    time = 'Time'
+    longitude = 'Longitude'
+    pixIndXtrackAxis = 1
+    outUnits = { 'Time' : 'second',
+                 'Latitude' : 'degree',
+                 'Longitude' :'degree (-180 to 180)',
+                 'SpacecraftLatitude' : 'degree', 
+                 'SpacecraftLongitude' : 'degree (-180 to 180)',
+                 'SpacecraftAltitude' : 'meters',
+                 'SolarZenithAngle' : 'degree',
+                 'SolarAzimuthAngle' : 'degree',
+                 'ViewingZenithAngle' : 'degree',
+                 'ViewingAzimuthAngle' : 'degree',
+                 'ColumnAmountNO2' : 'molecules/cm^2',
+                 'ColumnAmountNO2Std' : 'molecules/cm^2',
+                 'ColumnAmountNO2Strat' : 'molecules/cm^2',
+                 'ColumnAmountNO2StratStd' : 'molecules/cm^2',
+                 'ColumnAmountNO2Trop' : 'molecules/cm^2',
+                 'ColumnAmountNO2TropStd' : 'molecules/cm^2',
+                 'SlantColumnAmountNO2' : 'molecules/cm^2',
+                 'SlantColumnAmountNO2Std' : 'molecules/cm^2',
+                 'SlantColumnAmountNO2Destriped' : 'molecules/cm^2',
+                 'RingCoefficient' : 'NoUnits',
+                 'RingCoefficientStd' : 'NoUnits',
+                 'SlantColumnAmountO3' : 'molecules/cm^2',
+                 'SlantColumnAmountO3Std' : 'molecules/cm^2',
+                 'SlantColumnAmountH2O' : 'molecules/cm^2',
+                 'SlantColumnAmountH2Otd' : 'molecules/cm^2',
+                 'SlantColumnAmountO2O2' : 'molecules/cm^2',
+                 'SlantcolumnAmountO2O2Std' : 'molecules/cm^2',
+                 'PolynomialCoefficients' : 'NoUnits',
+                 'PolynomialCoefficientsStd' : 'NoUnits',
+                 'ChiSquaredOfFit' : 'NoUnits',
+                 'RootMeanSquareErrorOfFit' : 'sr^-1',
+                 'AmfQualityFlags' : 'NoUnits',
+                 'AlgorithmFlags' : 'NoUnits',
+                 'AmfStrat' : 'NoUnits',
+                 'AmfStratClear' : 'NoUnits',
+                 'AmfStratCloudy' : 'NoUnits',
+                 'AmfStratStd' : 'NoUnits',
+                 'AmfTrop' : 'NoUnits',
+                 'AmfTropClear' : 'NoUnits',
+                 'AmfTropCloudy' : 'NoUnits',
+                 'AmfTropStd' : 'NoUnits',
+                 'CloudFraction' : 'NoUnits',
+                 'CloudFractionStd' : 'NoUnits',
+                 'CloudRadianceFraction' : 'NoUnits',
+                 'CloudPressure' : 'hPa',
+                 'CloudPressureStd' : 'hPa',
+                 'TerrainReflectivity' : 'NoUnits',
+                 'TerrainPressure' : 'hPa',
+                 'TerrainHeight' : 'meters',
+                 'SmallPixelRadiancePointer' : 'NoUnits',
+                 'SmallPixelRadiance' : 'photons/ (s.nm.cm^2.sr)',
+                 'InstrumentConfigurationId' : 'NoUnits',
+                 'MeasurementQualityFlags' : 'NoUnits',
+                 'FitQualityFlags' : 'NoUnits',
+                 'WavelengthRegistrationCheck' : 'nm',
+                 'WavelengthRegistrationCheckStd' : 'nm',
+                 'UnpolFldLatBandQualityFlags' : 'NoUnits',
+                 'VcdQualityFlags' : 'NoUnits',
+                 'XTrackQualityFlags' : 'NoUnits',
+                 'ScatteringWeight' : 'NoUnits', 
+                 'ScatteringWtPressure' : 'hPa',
+                 'FoV75CornerLatitude' : 'degree',
+                 'FoV75CornerLongitude' :'degree(-180 to 180)'}
+
+    extraDimLabel = {'Time' : 'none',
+                 'Latitude' : 'none',
+                 'Longitude' :'none',
+                 'SpacecraftLatitude' : 'none', 
+                 'SpacecraftLongitude' : 'none',
+                 'SpacecraftAltitude' : 'none',
+                 'SolarZenithAngle' : 'none',
+                 'SolarAzimuthAngle' : 'none',
+                 'ViewingZenithAngle' : 'none',
+                 'ViewingAzimuthAngle' : 'none',
+                 'ColumnAmountNO2' : 'none',
+                 'ColumnAmountNO2Std' : 'none',
+                 'ColumnAmountNO2Strat' : 'none',
+                 'ColumnAmountNO2StratStd' : 'none',
+                 'ColumnAmountNO2Trop' : 'none',
+                 'ColumnAmountNO2TropStd' : 'none',
+                 'SlantColumnAmountNO2' : 'none',
+                 'SlantColumnAmountNO2Std' : 'none',
+                 'SlantColumnAmountNO2Destriped' : 'none',
+                 'RingCoefficient' : 'none',
+                 'RingCoefficientStd' : 'none',
+                 'SlantColumnAmountO3' : 'none',
+                 'SlantColumnAmountO3Std' : 'none',
+                 'SlantColumnAmountH2O' : 'none',
+                 'SlantColumnAmountH2Otd' : 'none',
+                 'SlantColumnAmountO2O2' : 'none',
+                 'SlantcolumnAmountO2O2Std' : 'none',
+                 'PolynomialCoefficients' : 'none',
+                 'PolynomialCoefficientsStd' : 'none',
+                 'ChiSquaredOfFit' : 'none',
+                 'RootMeanSquareErrorOfFit' : 'none',
+                 'AmfQualityFlags' : 'none',
+                 'AlgorithmFlags' : 'none',
+                 'AmfStrat' : 'none',
+                 'AmfStratClear' : 'none',
+                 'AmfStratCloudy' : 'none',
+                 'AmfStratStd' : 'none',
+                 'AmfTrop' : 'none',
+                 'AmfTropClear' : 'none',
+                 'AmfTropCloudy' : 'none',
+                 'AmfTropStd' : 'none',
+                 'CloudFraction' : 'none',
+                 'CloudFractionStd' : 'none',
+                 'CloudRadianceFraction' : 'none',
+                 'CloudPressure' : 'none',
+                 'CloudPressureStd' : 'none',
+                 'TerrainReflectivity' : 'none',
+                 'TerrainPressure' : 'none',
+                 'TerrainHeight' : 'none',
+                 'SmallPixelRadiancePointer' : 'SmallPixelPointer',
+                 'SmallPixelRadiance' : 'none',
+                 'InstrumentConfigurationId' : 'none',
+                 'MeasurementQualityFlags' : 'none',
+                 'FitQualityFlags' : 'none',
+                 'WavelengthRegistrationCheck' : 'WavelengthCheck',
+                 'WavelengthRegistrationCheckStd' : 'WavelengthCheck',
+                 'UnpolFldLatBandQualityFlags' : 'none',
+                 'VcdQualityFlags' : 'none',
+                 'XTrackQualityFlags' : 'none',
+                 'ScatteringWeight' : 'layer', 
+                 'ScatteringWtPressure' : 'none',
+                 'FoV75CornerLatitude' : 'corner',
+                 'FoV75CornerLongitude' :'corner'}
+                 	
+                 	
+
+    extraDimSize = { 'Time' : 0,
+                 'Latitude' : 0,
+                 'Longitude' :0,
+                 'SpacecraftLatitude' : 0, 
+                 'SpacecraftLongitude' : 0,
+                 'SpacecraftAltitude' : 0,
+                 'SolarZenithAngle' : 0,
+                 'SolarAzimuthAngle' : 0,
+                 'ViewingZenithAngle' : 0,
+                 'ViewingAzimuthAngle' : 0,
+                 'ColumnAmountNO2' : 0,
+                 'ColumnAmountNO2Std' : 0,
+                 'ColumnAmountNO2Strat' : 0,
+                 'ColumnAmountNO2StratStd' : 0,
+                 'ColumnAmountNO2Trop' : 0,
+                 'ColumnAmountNO2TropStd' : 0,
+                 'SlantColumnAmountNO2' : 0,
+                 'SlantColumnAmountNO2Std' : 0,
+                 'SlantColumnAmountNO2Destriped' : 0,
+                 'RingCoefficient' : 0,
+                 'RingCoefficientStd' : 0,
+                 'SlantColumnAmountO3' : 0,
+                 'SlantColumnAmountO3Std' : 0,
+                 'SlantColumnAmountH2O' : 0,
+                 'SlantColumnAmountH2Otd' : 0,
+                 'SlantColumnAmountO2O2' : 0,
+                 'SlantcolumnAmountO2O2Std' : 0,
+                 'PolynomialCoefficients' : 6,
+                 'PolynomialCoefficientsStd' : 6,
+                 'ChiSquaredOfFit' : 0,
+                 'RootMeanSquareErrorOfFit' : 0,
+                 'AmfQualityFlags' : 0,
+                 'AlgorithmFlags' : 0,
+                 'AmfStrat' : 0,
+                 'AmfStratClear' : 0,
+                 'AmfStratCloudy' : 0,
+                 'AmfStratStd' : 0,
+                 'AmfTrop' : 0,
+                 'AmfTropClear' : 0,
+                 'AmfTropCloudy' : 0,
+                 'AmfTropStd' : 0,
+                 'CloudFraction' : 0,
+                 'CloudFractionStd' : 0,
+                 'CloudRadianceFraction' : 0,
+                 'CloudPressure' : 0,
+                 'CloudPressureStd' : 0,
+                 'TerrainReflectivity' : 0,
+                 'TerrainPressure' : 0,
+                 'TerrainHeight' : 0,
+                 'SmallPixelRadiancePointer' : 2,
+                 'SmallPixelRadiance' : 0,
+                 'InstrumentConfigurationId' : 0,
+                 'MeasurementQualityFlags' : 0,
+                 'FitQualityFlags' : 10,
+                 'WavelengthRegistrationCheck' : 60,
+                 'WavelengthRegistrationCheckStd' : 60,
+                 'UnpolFldLatBandQualityFlags' : 0,
+                 'VcdQualityFlags' : 0,
+                 'XTrackQualityFlags' : 0,
+                 'ScatteringWeight' : 35, 
+                 'ScatteringWtPressure' : 0,
+                 'FoV75CornerLatitude' : 4,
+                 'FoV75CornerLongitude' :4}
+                 
+
