@@ -110,7 +110,10 @@ def getLongName(fPath):
     try:
         node = fid.getNode('/', 'HDFEOS INFORMATION/ArchiveMetadata')
     except tables.exceptions.NoSuchNodeError:
-        node = fid.getNode('/', 'HDFEOS INFORMATION/ArchiveMetadata.0')
+        try:
+            node = fid.getNode('/', 'HDFEOS INFORMATION/ArchiveMetadata.0')
+        except:
+            node = fid.getNode('/', 'HDFEOS INFORMATION/ArchivedMetadata.0')
     bigString = str(list(node)[0])
     strings = bigString.split('\n')
     for i in range(len(strings)):
@@ -716,7 +719,7 @@ class HDFnasaomil2_File(HDFFile):
                     'WavelengthRegistrationCheck' : __dataPath+'WavelengthRegistrationCheck',
                     'WavelengthRegistrationCheckStd' : __dataPath+'WavelengthRegistrationCheckStd',
                     'XTrackQualityFlags' : __dataPath+'XTrackQualityFlags',
-                    'vcdQualityFlags' : __dataPath+'vcdQualityFlags',
+                    'VcdQualityFlags' : __dataPath+'VcdQualityFlags',
                     'GroundPixelQualityFlags' : __geoPath+'GroundPixelQualityFlags',
                     'Latitude' : __geoPath+'Latitude',
                     'Longitude' : __geoPath+'Longitude',
@@ -1165,3 +1168,356 @@ class HDFmodisl2_File(HDF4File):
                                   ' use the point_in_cell mapping function '\
                                   'instead.')
 
+class HDFnasaomihchol2_File(HDFFile):
+    """
+    Provide interface to NASA OMI HCHO L2 product, with pixel corners
+    
+    Pixel corners are retrieved from an extra file that must be accessible
+    in cornerDir.  The files listed in cornerFileList, if any, will be checked
+    first, followed by any files in cornerDir. If no valid pixel corner file
+    is found that matches the orbit number of the input file, the parser will
+    instantiage but get_geo_corners will fail with an IOError. 
+
+    The corners retrieved are for the visible channel
+    used by the NO2 algorithm- using this parser for other products may 
+    require altering the parser to use a different channel if 
+    appropriate.
+    
+    The keys to retrieve variables are the names of the variables within actual
+    files.  Note that the NASA product documentation has the wrong name for 
+    the "SlantColumnAmountH20Std" variable due to a case typo.
+
+    Does not support the fields "UnpolFldCoefficients" or "SmallPixelRadiance"
+    because the dimensionality of these fields changes between files and there
+    is no way to deal with fields with variable dimension size in the current 
+    framework.
+    """
+    
+    OMIAURAHCHO_FILE_NAME = "OMI/Aura Formaldehyde (HCHO) Total "\
+                           "Column 1-Orbit L2 Swath 13x24km"
+    OMIAURAHCHO_CORNER_FILE_NAME = "OMI/Aura Global Ground Pixel Corners "\
+                                  "1-Orbit L2 Swath 13x24km"
+
+    def __init__(self, filename, subtype='', extension=None, cornerDir=None,
+                 cornerFileList=None):
+        HDFFile.__init__(self, filename, subtype, extension)
+
+        # make sure filename is actually an input file
+        if getLongName(filename) != HDFnasaomihchol2_File.OMIAURAHCHO_FILE_NAME:
+            raise IOError('Attempt to read non-NASA OMI L2 file as such.')
+        
+        # start by assuming we aren't going to find anything
+        self.pixCorners = None
+
+        # see if the corner directory even exists.  If it doesn't, we obviously can't 
+        # find a corner file
+        if os.path.isdir(cornerDir):
+
+            # convert the corner files into full pathnames
+            # unless we were given null string (signal to search directory)
+            if cornerFileList != ['']:
+                cornerFileList = [os.path.join(cornerDir, f) for f in cornerFileList]
+
+            # get orbit number of file for matching
+            forbitnumber = getOrbitNumber(filename)
+
+            # try using the list
+        
+            for f in cornerFileList:
+                if f != '' and getLongName(f) == HDFnasaomihchol2_File.OMIAURAHCHO_CORNER_FILE_NAME:
+                    try:
+                        if getOrbitNumber(f) == forbitnumber:
+                            self.pixCorners = f
+                            break
+                    except:
+                        pass
+
+            # if necessary, search entire corner file directory
+            if self.pixCorners == None:        
+                allPossible = [os.path.join(cornerDir, f) for f in os.listdir(cornerDir)]
+                for f in allPossible:
+                    try:
+                        if tables.isHDF5File(f) \
+                                and getLongName(f) == HDFnasaomihchol2_File.OMIAURAHCHO_CORNER_FILE_NAME \
+                                and getOrbitNumber(f) == forbitnumber:
+                            self.pixCorners = f
+                            break
+                    except:
+                        pass
+                
+        if self.pixCorners == None:
+            print "No valid corner file found for {0}.".format(filename)                        
+            
+    __dataPath = '/HDFEOS/SWATHS/OMI Total Column Amount HCHO/Data Fields/'
+    __geoPath = '/HDFEOS/SWATHS/OMI Total Column Amount HCHO/Geolocation Fields/'
+    
+    _nameExpMap = {'GroundPixelQualityFlags' : __dataPath+'GroundPixelQualityFlags',
+                   'ColumnAmount' : __dataPath+'ColumnAmount',
+                   'ColumnAmountDestriped' :__dataPath+'ColumnAmountDestriped',
+                   'ColumnAmountUncertainty' :__dataPath+'ColumnAmountUncertainty' ,
+                   'ClimatologyLevels' :__dataPath+'ClimatologyLevels',
+                   'ColumnAmount' :__dataPath+'ColumnAmount',
+                   'ColumnAmountDestriped' :__dataPath+'ColumnAmountDestriped',
+                   'ColumnUncertainty' :__dataPath+'ColumnUncertainty',
+                   'GasProfile' :__dataPath+'GasProfile',
+                   'RadianceReferenceColumnAmount' :__dataPath+'RadianceReferenceColumnAmount',
+                   'RadianceReferenceColumnUncertainty' :__dataPath + 'RadianceReferenceColumnUncertainty',
+                   'ReferenceSectorCorrectedVerticalColumn' :__dataPath+'ReferenceSectorCorrectedVerticalColumn',
+                   'ScatteringWeights' :__dataPath+'ScatteringWeights',
+                   'SolarWavCalConvergenceFlag' :__dataPath+'SolarWavCalConvergenceFlag',
+                   'AMFCloudFraction' :__dataPath+'AMFCloudFraction',
+                   'AMFCloudPressure' :__dataPath+'AMFCloudPressure',
+                   'AirMassFactor' :__dataPath+'AirMassFactor',
+                   'AirMassFactorDiagnosticFlag' :__dataPath+'AirMassFactorDiagnosticFlag',
+                   'AirMassFactorGeometric' :__dataPath+'AirMassFactorGeometric',
+                   'Albedo' :__dataPath+'Albedo',
+                   'PixelCornerLongitudes' :__dataPath+'PixelCornerLongitudes',
+                   'PixelCornerLatitudes' :__dataPath+'PixelCornerLatitudes',                   	
+                   'MainDataQualityFlag' :__dataPath+'MainDataQualityFlag',
+                   'FitConvergenceFlag' : __dataPath+'FitConvergenceFlag',
+                   'RadianceReferenceConvergenceFlag' : __dataPath+'RadianceReferenceConvergenceFlag',
+                   'RadianceWavCalConvergeFlag' : __dataPath+'RadianceWavCalConvergeFlag',
+                   'XtrackQualityFlags' : __geoPath+'XtrackQualityFlags',
+                   'Latitude' : __geoPath+'Latitude',
+                   'Longitude' : __geoPath+'Longitude',
+                   'TerrainHeight' :__geoPath+'TerrainHeight',
+                   'SolarAzimuthAngle' : __geoPath+'SolarAzimuthAngle',
+                   'SolarZenithAngle' : __geoPath+'SolarZenithAngle',
+                   'SpacecraftAltitude' : __geoPath+'SpacecraftAltitude',
+                   'Time' : __geoPath+'Time',
+                   'ViewingAzimuthAngle' : __geoPath+'ViewingAzimuthAngle',
+                   'ViewingZenithAngle' : __geoPath+'ViewingZenithAngle'}
+    
+    _indexMap = {'default' : lambda var, ind: var[..., ind[0], ind[1]],
+    	          'Time' : lambda var, ind: var[ind[0]]}
+                  	    
+    
+    def get_geo_corners(self):
+        '''
+        Retrieves array of the corners of the pixels.  
+        
+        Throws IOError if no pixel corner file specified
+        '''
+        latNodeName = '/HDFEOS/SWATHS/OMI Ground Pixel Corners VIS/Data Fields/FoV75CornerLatitude'
+        lonNodeName = '/HDFEOS/SWATHS/OMI Ground Pixel Corners VIS/Data Fields/FoV75CornerLongitude'
+        try:
+            pxFid = tables.openFile(self.pixCorners)
+        except AttributeError:
+            raise IOError('Unable to open pixel corners file.  Need pixel corners file to use corners')
+        try:
+            latNode = pxFid.getNode('/', latNodeName)
+            lonNode = pxFid.getNode('/', lonNodeName)
+            # Note: it is assumed that there are no missing values.
+            lat = latNode[:].transpose((1,2,0))
+            lon = lonNode[:].transpose((1,2,0))            
+        finally:
+            pxFid.close()
+        ind = numpy.indices(lat.shape[0:2]).transpose((1,2,0))
+        protoDtype = [('lat', lat.dtype, 4), ('lon', lon.dtype, 4), ('ind', ind.dtype, 2)]
+        struct = numpy.zeros(lat.shape[0:2], dtype=protoDtype)
+        (struct['lat'], struct['lon'], struct['ind']) = (lat, lon, ind)
+        return struct
+    
+    def get_geo_centers(self):
+        lat = self.get('Latitude')
+        lon = self.get('Longitude')
+        ind = numpy.indices(lat.shape).transpose((1,2,0))
+        protoDtype = [('lat', lat.dtype), ('lon', lon.dtype), ('ind', ind.dtype, 2)]
+        struct = numpy.zeros(lat.shape, dtype=protoDtype)
+        (struct['lat'], struct['lon'], struct['ind']) = (lat, lon, ind)
+        return struct
+
+class HDFnasaomiv3l2_File(HDFFile):
+    """
+    Provide interface to NASA OMI L2 product, with pixel corners
+    
+    Pixel corners are retrieved from an extra file that must be accessible
+    in cornerDir.  The files listed in cornerFileList, if any, will be checked
+    first, followed by any files in cornerDir. If no valid pixel corner file
+    is found that matches the orbit number of the input file, the parser will
+    instantiage but get_geo_corners will fail with an IOError. 
+
+    The corners retrieved are for the visible channel
+    used by the NO2 algorithm- using this parser for other products may 
+    require altering the parser to use a different channel if 
+    appropriate.
+    
+    The keys to retrieve variables are the names of the variables within actual
+    files.  Note that the NASA product documentation has the wrong name for 
+    the "SlantColumnAmountH20Std" variable due to a case typo.
+
+    Does not support the fields "UnpolFldCoefficients" or "SmallPixelRadiance"
+    because the dimensionality of these fields changes between files and there
+    is no way to deal with fields with variable dimension size in the current 
+    framework.
+    """
+    
+    OMIAURANO2_FILE_NAME = "OMI/Aura Nitrogen Dioxide (NO2) Total & "\
+                           "Troposph.Column 1-Orbit L2 Swath 13x24km"
+    OMIAURANO2_CORNER_FILE_NAME = "OMI/Aura Global Ground Pixel Corners "\
+                                  "1-Orbit L2 Swath 13x24km"
+
+    def __init__(self, filename, subtype='', extension=None, cornerDir=None,
+                 cornerFileList=None):
+        HDFFile.__init__(self, filename, subtype, extension)
+
+        # make sure filename is actually an input file
+        if getLongName(filename) != HDFnasaomiv3l2_File.OMIAURANO2_FILE_NAME:
+            raise IOError('Attempt to read non-NASA OMI L2 file as such.')
+        
+        # start by assuming we aren't going to find anything
+        self.pixCorners = None
+
+        # see if the corner directory even exists.  If it doesn't, we obviously can't 
+        # find a corner file
+        if os.path.isdir(cornerDir):
+
+            # convert the corner files into full pathnames
+            # unless we were given null string (signal to search directory)
+            if cornerFileList != ['']:
+                cornerFileList = [os.path.join(cornerDir, f) for f in cornerFileList]
+
+            # get orbit number of file for matching
+            forbitnumber = getOrbitNumber(filename)
+
+            # try using the list
+        
+            for f in cornerFileList:
+                if f != '' and getLongName(f) == HDFnasaomiv3l2_File.OMIAURANO2_CORNER_FILE_NAME:
+                    try:
+                        if getOrbitNumber(f) == forbitnumber:
+                            self.pixCorners = f
+                            break
+                    except:
+                        pass
+
+            # if necessary, search entire corner file directory
+            if self.pixCorners == None:        
+                allPossible = [os.path.join(cornerDir, f) for f in os.listdir(cornerDir)]
+                for f in allPossible:
+                    try:
+                        if tables.isHDF5File(f) \
+                                and getLongName(f) == HDFnasaomil2_File.OMIAURANO2_CORNER_FILE_NAME \
+                                and getOrbitNumber(f) == forbitnumber:
+                            self.pixCorners = f
+                            break
+                    except:
+                        pass
+                
+        if self.pixCorners == None:
+            print "No valid corner file found for {0}.".format(filename)              
+    __dataPath = '/HDFEOS/SWATHS/ColumnAmountNO2/Data Fields/'
+    __geoPath = '/HDFEOS/SWATHS/ColumnAmountNO2/Geolocation Fields/'
+    
+    _nameExpMap = {'Time' : __geoPath+'Time',
+                 'Latitude' : __geoPath+'Latitude',
+                 'Longitude' :__geoPath+'Longitude',
+                 'SpacecraftLatitude' : __geoPath+'SpacecraftLatitude', 
+                 'SpacecraftLongitude' : __geoPath+'SpacecraftLongitude',
+                 'SpacecraftAltitude' : __geoPath+'SpacecraftAltitude',
+                 'SolarZenithAngle' : __geoPath+'SolarZenithAngle',
+                 'SolarAzimuthAngle' : __geoPath+'SolarAzimuthAngle',
+                 'ViewingZenithAngle' : __geoPath+'ViewingZenithAngle',
+                 'ViewingAzimuthAngle' : __geoPath+'ViewingAzimuthAngle',
+                 'ColumnAmountNO2' : __dataPath+'ColumnAmountNO2',
+                 'ColumnAmountNO2Std' : __dataPath+'ColumnAmountNO2Std',
+                 'ColumnAmountNO2Strat' : __dataPath+'ColumnAmountNO2Strat' ,
+                 'ColumnAmountNO2StratStd' : __dataPath+'ColumnAmountNO2StratStd',
+                 'ColumnAmountNO2Trop' : __dataPath+'ColumnAmountNO2Trop',
+                 'ColumnAmountNO2TropStd' : __dataPath+'ColumnAmountNO2TropStd',
+                 'SlantColumnAmountNO2' : __dataPath+'SlantColumnAmountNO2',
+                 'SlantColumnAmountNO2Std' : __dataPath+'SlantColumnAmountNO2Std',
+                 'SlantColumnAmountNO2Destriped' : __dataPath+'SlantColumnAmountNO2Std',
+                 'RingCoefficient' : __dataPath+'RingCoefficient',
+                 'RingCoefficientStd' : __dataPath+'RingCoefficientStd',
+                 'SlantColumnAmountO3' : __dataPath+'SlantColumnAmountO3',
+                 'SlantColumnAmountO3Std' : __dataPath+'SlantColumnAmountO3Std',
+                 'SlantColumnAmountH2O' : __dataPath+'SlantColumnAmountH2O',
+                 'SlantColumnAmountH2Otd' : __dataPath+'SlantColumnAmountH2Otd',
+                 'SlantColumnAmountO2O2' : __dataPath+'SlantColumnAmountO2O2',
+                 'SlantcolumnAmountO2O2Std' : __dataPath+'SlantcolumnAmountO2O2Std',
+                 'PolynomialCoefficients' : __dataPath +'PolynomialCoefficients',
+                 'PolynomialCoefficientsStd' : __dataPath + 'PolynomialCoefficientsStd',
+                 'ChiSquaredOfFit' : __dataPath+'ChiSquaredOfFit',
+                 'RootMeanSquareErrorOfFit' : __dataPath+'RootMeanSquareErrorOfFit',
+                 'AmfQualityFlags' : __dataPath+'AmfQualityFlags',
+                 'AlgorithmFlags' : __dataPath+'AlgorithmFlags',
+                 'AmfStrat' : __dataPath+'AmfStrat',
+                 'AmfStratClear' : __dataPath+'AmfStratClear',
+                 'AmfStratCloudy' : __dataPath+'AmfStratCloudy',
+                 'AmfStratStd' : __dataPath+ 'AmfStratStd',
+                 'AmfTrop' : __dataPath+'AmfTrop',
+                 'AmfTropClear' : __dataPath+'AmfTropClear',
+                 'AmfTropCloudy' : __dataPath+'AmfTropCloudy',
+                 'AmfTropStd' : __dataPath+'AmfTropStd',
+                 'CloudFraction' : __dataPath+'CloudFraction' ,
+                 'CloudFractionStd' : __dataPath+'CloudFractionStd',
+                 'CloudRadianceFraction' : __dataPath+'CloudRadianceFraction',
+                 'CloudPressure' : __dataPath+'CloudPressure',
+                 'CloudPressureStd' : __dataPath+'CloudPressureStd',
+                 'TerrainReflectivity' : __dataPath+'TerrainReflectivity',
+                 'TerrainPressure' : __dataPath+'TerrainPressure',
+                 'TerrainHeight' : __dataPath+'TerrainHeight' ,
+                 'SmallPixelRadiancePointer' :__dataPath +'SmallPixelRadiancePointer',
+                 'SmallPixelRadiance' : __dataPath+'SmallPixelRadiance',
+                 'InstrumentConfigurationId' : __dataPath+'InstrumentConfigurationId',
+                 'MeasurementQualityFlags' : __dataPath+'MeasurementQualityFlags',
+                 'FitQualityFlags' : __dataPath+'FitQualityFlags',
+                 'WavelengthRegistrationCheck' : __dataPath+ 'WavelengthRegistrationCheck',
+                 'WavelengthRegistrationCheckStd' : __dataPath+'UnpolFldLatBandQualityFlags',
+                 'UnpolFldLatBandQualityFlags' : __dataPath+'UnpolFldLatBandQualityFlags',
+                 'VcdQualityFlags' : __dataPath+'VcdQualityFlags',
+                 'XTrackQualityFlags' : __dataPath+'XTrackQualityFlags',
+                 'ScatteringWeight' : __dataPath+ 'ScatteringWeight', 
+                 'ScatteringWtPressure' : __dataPath+'ScatteringWtPressure',
+                 'FoV75CornerLatitude' :__geoPath+'FoV75CornerLatitude',
+                 'FoV75CornerLongitude' :__geoPath+'FoV75CornerLongitude'}
+                 
+    
+    _indexMap = {'default' : lambda var, ind: var[ind[0], ind[1], ...],
+                  'SmallPixelRadiance' : lambda var, ind: var[:, ind[1]],
+                  'SmallPixelRadiancePointer' : lambda var, ind: var[ind[0], :],
+                  'InstrumentConfigurationId' : lambda var, ind: var[ind[0]],
+                  'MeasurementQualityFlags' : lambda var, ind: var[ind[0]],
+                  'WavelengthRegistrationCheck' : lambda var, ind: var[ind[0], :],
+                  'WavelengthRegistrationCheckStd' : lambda var, ind: var[ind[0], :],
+                  'UnpolFldLatBandQualityFlags' : lambda var, ind: var[:],
+                  'Time' : lambda var, ind: var[ind[0]],
+                  'SpacecraftLatitude' : lambda var, ind: var[ind[0]],
+                  'SpacecraftLongitude' : lambda var, ind: var[ind[0]],
+                  'SpacecraftAltitude' : lambda var, ind: var[ind[0]],
+                  'ScatteringWtPressure' : lambda var, ind: var[:]}
+    
+    def get_geo_corners(self):
+        '''
+        Retrieves array of the corners of the pixels.  
+        
+        Throws IOError if no pixel corner file specified
+        '''
+        latNodeName = '/HDFEOS/SWATHS/OMI Ground Pixel Corners VIS/Data Fields/FoV75CornerLatitude'
+        lonNodeName = '/HDFEOS/SWATHS/OMI Ground Pixel Corners VIS/Data Fields/FoV75CornerLongitude'
+        try:
+            pxFid = tables.openFile(self.pixCorners)
+        except AttributeError:
+            raise IOError('Unable to open pixel corners file.  Need pixel corners file to use corners')
+        try:
+            latNode = pxFid.getNode('/', latNodeName)
+            lonNode = pxFid.getNode('/', lonNodeName)
+            # Note: it is assumed that there are no missing values.
+            lat = latNode[:].transpose((1,2,0))
+            lon = lonNode[:].transpose((1,2,0))            
+        finally:
+            pxFid.close()
+        ind = numpy.indices(lat.shape[0:2]).transpose((1,2,0))
+        protoDtype = [('lat', lat.dtype, 4), ('lon', lon.dtype, 4), ('ind', ind.dtype, 2)]
+        struct = numpy.zeros(lat.shape[0:2], dtype=protoDtype)
+        (struct['lat'], struct['lon'], struct['ind']) = (lat, lon, ind)
+        return struct
+    
+    def get_geo_centers(self):
+        lat = self.get('Latitude')
+        lon = self.get('Longitude')
+        ind = numpy.indices(lat.shape).transpose((1,2,0))
+        protoDtype = [('lat', lat.dtype), ('lon', lon.dtype), ('ind', ind.dtype, 2)]
+        struct = numpy.zeros(lat.shape, dtype=protoDtype)
+        (struct['lat'], struct['lon'], struct['ind']) = (lat, lon, ind)
\ No newline at end of file
