@@ -7,7 +7,7 @@ Process a series of files, generating some kind of output for each
 If verbose is set to True, all default status updates will be printed.  
 If set to False, the program will run silently
 
-@version 10/5/2012
+@version 11/21/2014
 @author: maki, oberman
 '''
 import os
@@ -24,11 +24,12 @@ from process_sat import map_geo
 from process_sat import out_geo
 from process_sat import utils
 from process_sat import filetypes
+#import map_geo
 
 '''
 VERSION NUMBER
 '''
-__version__ = "1.1.6"
+__version__ = "2.0.0"
 
 class NeedToParseInFileException(Exception):
     '''exception class for signaling the need to parse input file'''
@@ -206,14 +207,13 @@ parser.add_argument('--includeGrid', metavar='GridFileName', \
                     'write out the latitudes and longitudes of the gridcells '\
                     'defined by the selected projection')
 parser.add_argument('--verbose', help='Supply False here to disable ' \
-                    'verbose execution', default=True, choices={'True',\
-                    'False'})
+                    'verbose execution', default=True, choices=['True','False'])
 parser.add_argument('--interactive', help='Supply True here to enable ' \
                     'interactive error handling in the event that the ' \
                     'program encounters an invalid input file. (Default: ' \
                     'False ignores any invalid files and continues ' \
                     'processing all requested files)', default=False, \
-                    choices = {'True','False'}) 
+                    choices = ['True','False']) 
 parser.add_argument('--AttributeHelp', nargs='*', help='Supply this flag, ' \
                     'followed by a list of one or more projection names, ' \
                     'output function names, or filetypes to see a list of ' \
@@ -235,6 +235,7 @@ print "\n\n           WISCONSIN HORIZONTAL INTERPOLATION PROGRAM " \
       "FOR SATELLITES \n                                 Version " + \
       __version__ + "\n"
 
+
 try:
     gnomespice = parser.parse_args()
 except NeedToParseInFileException as fname:
